

* EXAMPLE 1 ... create a data set with some missing data ... look at PROC FREQ defaults;
data class;
set sashelp.class;
if ranuni(987) le .5;
if ranuni(987) le .2 then
   call missing(weight);
if ranuni(987) le .1 then 
   call missing(sex, age);
run;

title 'DATA SET CLASS ... ALL VARIABLES';
proc freq data=class;
run;

title 'DATA SET CLASS ... ALL VARIABLES + MISSING OPTION';
proc freq data=class;
tables _all_ / missing;
run;
title;

* EXAMPLE 2 ... formats for grouping data values;
proc format;
value nm low-high = 'OK' other = 'MISSING';
value $ch  ' ' = 'MISSING'  other = 'OK'
;
run;

* EXAMPLE 3 ... all tables to OUTPUT window;
title 'DATA SET CLASS ... ALL VARIABLES ... FORMATTED INTO TWO GROUPS';
proc freq data=class;
tables _all_ / missing;
format _numeric_ nm.  _character_ $ch.;
run;
title;

* EXAMPLE 4 ... last table to data set TABLES;
proc freq data=class;
tables _all_ / noprint missing out=tables; 
format _numeric_ nm. _character_ $ch.;
run;

title 'DATA SET TABLES ... USING PROC FREQ OUTPUT';
proc print data=tables noobs;
run;
title;

proc print data=tables noobs;
format _all_;
run;
title;

* EXAMPLE 5 ... all tables to data set TABLES;
proc format;
value nm  low-high = '1' other = '0';
value $ch ' ' = '0' other = '1';
run;

ods listing close;
ods output onewayfreqs=tables;

proc freq data=class;
tables _all_ / missing;
format _numeric_ nm. _character_ $ch.;
run;

ods output close;
ods listing;

title 'DATA SET TABLES ... USING ODS OUTPUT';
proc print data=tables noobs;
format _all_;
run;
title;

* EXAMPLE 6 ... parse results in data set TABLES, create a report;
data report;
length var $32; 

do until (last.table); 
   set tables; 
   by table notsorted;
   array names(*) f_: ;
   select (names(_n_)); 
     when ('0') do; 
        miss   = frequency; 
        p_miss = percent;
     end;
     when ('1') do; 
        ok   = frequency; 
        p_ok = percent;  
     end;
   end;
end;

miss   = coalesce(miss,0); 
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);

var    = scan(table,-1); 

keep var miss ok p_: ; 
format miss ok comma7. p_: 5.1;
label
miss   = 'N_MISSING'
ok     = 'N_OK'
p_miss = '%_MISSING'
p_ok   = '%_OK'
var    = 'VARIABLE'
;
run;

title 'DATA SET CLASS';
proc print data=report noobs;
run;
title;

* use another data set ... SASHELP.HEART ... no other changes;

ods listing close;
ods output onewayfreqs=tables;

proc freq data=sashelp.heart;
tables _all_ / missing;
format _numeric_ nm. _character_ $ch.;
run;

ods output close;
ods listing;

data report;
length var $32; 

do until (last.table); 
   set tables; 
   by table notsorted;
   array names(*) f_: ;
   select (names(_n_)); 
     when ('0') do; 
        miss   = frequency; 
        p_miss = percent;
     end;
     when ('1') do; 
        ok   = frequency; 
        p_ok = percent;  
     end;
   end;
end;

miss   = coalesce(miss,0); 
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);

var    = scan(table,-1); 

keep var miss ok p_: ; 
format miss ok comma7. p_: 5.1;
label
miss   = 'N_MISSING'
ok     = 'N_OK'
p_miss = '%_MISSING'
p_ok   = '%_OK'
var    = 'VARIABLE'
;
run;
 
title 'DATA SET SASHELP.HEART';
proc print data=report noobs;
run;
title;

* add variable labels;

data report;
* add LABEL to LENGTH statement;
length var $32 label $100; 

do until (last.table); 
   set tables; 
   by table notsorted;
   array names(*) f_: ;
   select (names(_n_)); 
     when ('0') do; 
        miss   = frequency; 
        p_miss = percent;
     end;
     when ('1') do; 
        ok   = frequency; 
        p_ok = percent;  
     end;
   end;
end;

miss   = coalesce(miss,0); 
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);

var    = scan(table,-1);
* added statement;
label = vlabelx(var);

keep var label miss ok p_: ;

format miss ok comma7. p_: 5.1;
label
miss   = 'N_MISSING'
ok     = 'N_OK'
p_miss = '%_MISSING'
p_ok   = '%_OK'
var    = 'VARIABLE'
label  = 'LABEL'
;
run;

proc print data=report noobs;
run;

* "nicer looking" report;
ods results off;
ods listing close;
ods html file='z:\heart.html' style=barrettsblue;

title 'DATA SET SASHELP.HEART';
proc print data=report label noobs;
id var label;
run;
title;

ods html close;
ods listing;
ods results;

*EXAMPLE 7 ... PROC MI for patterns of missing numeric data;
data class;
set sashelp.class;
if ranuni(987) le .5;
if ranuni(987) le .2 then
   call missing(weight);
if ranuni(987) le .1 then 
   call missing(sex, age);
run;

title 'MISSING NUMERICS IN DATA SET CLASS ... PROC MI';
proc mi data=class;
ods select misspattern;
run;
title;

proc format;
value nm  low-high = 'X' other = '.' ;
value $ch ' ' = '.' other = 'X';
run;

title 'ALL MISSING DATA IN DATA SET CLASS ... PROC FREQ';
proc freq data=class;
table Name*Sex*Age*Height*Weight / list missing nocum;
format _numeric_ nm. _character_ $ch.;
run;
title;

* use PROC SQL to create the variable list;
proc sql noprint;
select name into :vars separated by '*'
from dictionary.columns
where libname eq 'WORK' and memname eq 'CLASS' ;
quit;

title 'ALL MISSING DATA IN DATA SET CLASS ... PROC FREQ + PROC SQL';
proc freq data=class;
tables &vars / list missing nocum out=miss;
format _numeric_ nm. _character_ $ch.;
run;
title;

*EXAMPLE 8 ... use data set REPORT from previous analysis of data set SASHELP.HEART;
* drop variables with 50%+ missing data;
proc sql noprint;
select var into :droplist separated by ' '
from report 
where p_miss ge 50;
quit;

%put &droplist;

data heart;
set sashelp.heart(drop=&droplist);
run;

*EXAMPLE 9 ... delete an observation if 5 or more variables are missing;
data heart;
set sashelp.heart;
if cmiss (of _all_) ge 5 then delete;
run;

*EXAMPLE 10 ... delete an observation if all variables are missing;
data old;
input name : $5. age zip : $5.;
datalines;
mike 64 12203
. . .
sara 35 .
;
run;

options missing=' ';
data new;
set old;
if missing(cats(of _all_)) then delete;
run;
options missing='.';

* why use OPTIONS MISSING;
data new;
length string $50;
set old;
string = cats(of _all_);
run;

title 'STRING = CONCATENATION OF ALL VARIABLE VALUES ... LOOK FOR EXTRA PERIODS';
proc print data=new;
run;
title;

data class;
set sashelp.class;
if ranuni(987) le .5;
if ranuni(987) le .2 then
   call missing(weight);
if ranuni(987) le .1 then 
   call missing(sex, age);
run;

data new;
length string $50;
set class;
string = cats(of _all_);
run;

title 'STRING = CONCATENATION OF ALL VARIABLE VALUES ... LOOK FOR EXTRA PERIODS';
proc print data=new;
run;
title;

* NLEVELS option on PROC FREQ;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . .Z 54 .H
MARK . 45 10 .H
;

title 'DATA SET TEST ... notice multiple levels of missing data, variable WEIGHT';
proc print data=test;
run;
title;

ods listing close;
ods output nlevels=tables;
proc freq data=test nlevels;
run;
ods output close;
ods listing;

title 'DATA SET TABLES ... PROC FREQ + NLEVELS';
proc print data=tables;
run;
title;

* what if no missing data;
data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE M 21 72 200
MARY F 56 54 120
MARK M 45 60 110
;

ods listing close;
ods output nlevels=tables;
proc freq data=test nlevels;
run;
ods output close;
ods listing;

title 'DATA SET TABLES ... PROC FREQ + NLEVELS ... no missing data';
proc print data=tables;
run;
title;

* create two groups of data;
proc format;
value nm ._ - .z = . other = 'OK' ;   /* complete range of possible missing data ._ through .z */
value $ch ' ' = ' ' other = 'OK';
run;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . .Z 54 .H
MARK . 45 10 .H
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

ods listing close;
ods output nlevels=tables;
proc freq data=test nlevels;
format _numeric_ nm. _character_ $ch.;
run;
ods output close;
ods listing;

title 'DATA SET TABLES ... ODS OUTPUT NLEVELS';
proc print data=tables;
run;
title;

proc sql noprint;
select tablevar into :droplist separated by ' '
from tables
where nnonmisslevels eq 0;
quit;

data new;
set test (drop=&droplist);
run;

title 'DATA SET NEW ... VARIABLE WEIGHT DROPPED ... ALL DATA MISSING';
proc print data=new;
run;
title;

* what if no missing data;
data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE M 21 72 200
MARY F 56 54 120
MARK M 45 60 110
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

ods listing close;
ods output nlevels=tables;
proc freq data=test nlevels;
format _numeric_ nm. _character_ $ch.;
run;
ods output close;
ods listing;

title 'DATA SET TABLES ... ODS OUTPUT NLEVELS';
proc print data=tables;
run;
title;

* check the LOG for an ERROR;
proc sql noprint;
select tablevar into :droplist separated by ' '
from tables
where nnonmisslevels eq 0;
quit;

* replace all missing numeric values with zero;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . .Z 54 .H
MARK . 45 10 .H
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

* use an array;

data test;
set test;
array nm(*) _numeric_;
do _n_ = 1 to dim(nm);
nm(_n_) + 0;
end;
run;

title 'DATA SET TEST';
proc print data=test;
run;
title;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . .Z 54 .H
MARK . 45 10 .H
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

proc stdize data=test out=new reponly missing=0;
run;

title 'DATA SET NEW';
proc print data=new;
run;
title;

* what values were replaced;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . 0 . .H
MARK . 45 10 0
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

proc format;
value fix low-high='0' other='1';
run;

filename nosee dummy;
data test;
file nosee;
set test;
put (_numeric_) (fix.) @;
missing = _file_;
put;
run;

proc stdize data=test out=new reponly missing=0;
run;

title 'DATA SET NEW ... THERE IS A NEW VARIABLE ... MISSING (1 IF ZERO REPLACED MISSING)';
proc print data=new;
run;
title;

* SUBSTITUTE THE MEDIAN;

data test;
input name : $10. gender : $1. age height weight;
datalines;
MIKE . 21 60 .L
MARY . . . .H
MARK . 45 10 .
;

title 'DATA SET TEST';
proc print data=test;
run;
title;

filename nosee dummy;
data test;
file nosee;
set test;
put (_numeric_) (fix.) @;
missing = _file_;
put;
run;

proc stdize data=test out=new reponly missing=median;
run;

title 'DATA SET NEW ... VARIABLE WEIGHT / IF ALL MISSING AND STDIZE CANNOT REPLACE, STILL SEE A 1';
proc print data=new;
run;
title;

* replace all missing character values with 'N/A';

data mike;
input name : $3. zip : $5. x y;
datalines;
aqs 12203 . .
. . 9 9
qwe 02138 . 1000
mmm . 99 .
;
run;

title 'DATA SET MIKE';
proc print data=mike;
run;
title;

data new;
set mike;
array ch(*) _character_;
do _n_ = 1 to dim(ch);
  ch(_n_) = coalescec(ch(_n_) , 'N/A');
end;
run; 

title 'DATA SET NEW';
proc print data=new;
run;
title;


