
* missing data macro;

%macro miss_report(din, fout, label=no);
* formats to group observations;
proc format;
value nm  low-high = '1'   other = '0';
value $ch ' ' = '0' other = '1';
run;

* PROC FREQ produces data set TABLES with counts of missing/non-missing values;
ods listing close;
ods output onewayfreqs=tables;

proc freq data=&din;
tables _all_ / missing;
format _numeric_ nm. _character_ $ch.;
run;

ods output close;
ods listing;

* create the missing data report as a data set;
data report;
length var $32 label $100;
do until (last.table);
   set tables;
   by table notsorted;
   array names(*) f_: ;
   select (names(_n_));
     when ('0') do; miss = frequency; p_miss = percent; end;
     when ('1') do; ok   = frequency; p_ok   = percent; end;
   end;
end;

miss   = coalesce(miss,0); 
ok     = coalesce(ok,0);
p_miss = coalesce(p_miss,0);
p_ok   = coalesce(p_ok,0);

var   = scan(table,-1);
label = vlabelx(var);

keep var label miss ok p_:;
format miss ok comma7. p_: 5.1;
label
miss   = 'N_MISSING'
ok     = 'N_OK'
p_miss = '%_MISSING'
p_ok   = '%_OK'
var    = 'VARIABLE'
label  = 'LABEL'
;
run;

* print the report ... output directed to an HTML file;
ods listing close;
ods results off;
ods html file="&fout" style=barrettsblue;

proc print data=report label noobs;
%if &label ne no %then %do;
    id var label; 
    var miss p_miss ok p_ok;
%end;
%else %do;
    id var; 
    var miss p_miss ok p_ok;
%end;
run;

ods html close;
ods results;
ods listing;
%mend;


*examples;

%miss_report(sashelp.class, z:\class.html);

%miss_report(sashelp.heart, z:\heart.html , label=yes)

%miss_report(sashelp.zipcode, z:\zipcode.html, label=yes);
